@@ -23,9 +23,9 @@ class FogNode(Node):
         # TODO Shutdown!
         global _fog_nodes_created
         super().__init__(f"fog_{_fog_nodes_created}", cu=FOG_CU,
-                         power_model=PowerModelNode(max_power=FOG_MAX_POWER, static_power=FOG_STATIC_POWER))
+                         power_model=PowerModelNode(max_power=FOG_MAX_POWER, static_power=FOG_STATIC_POWER),
+                         location=location)
         _fog_nodes_created += 1
-        self.location = location
         self.shutdown = FOG_IDLE_SHUTDOWN
 
     def measure_power(self) -> PowerMeasurement:
@@ -48,9 +48,8 @@ class FogNode(Node):
 class TrafficLight(Node):
     def __init__(self, location: "Location", application_sink: Node):
         global _traffic_lights_created
-        super().__init__(f"traffic_light_{_traffic_lights_created}", cu=0, power_model=PowerModelNode(0, 0))
+        super().__init__(f"traffic_light_{_traffic_lights_created}", location=location)
         _traffic_lights_created += 1
-        self.location = location
         self.application = self._create_cctv_application(application_sink)
 
     def _create_cctv_application(self, application_sink: Node):
@@ -67,7 +66,7 @@ class TrafficLight(Node):
 class Taxi(Node):
     def __init__(self, env: simpy.Environment, mobility_model: "TaxiMobilityModel", application_sinks: List[Node]):
         global _taxis_created
-        super().__init__(f"taxi_{_taxis_created}", cu=0, power_model=PowerModelNode(0, 0))
+        super().__init__(f"taxi_{_taxis_created}")
         _taxis_created += 1
         self.env = env
         self.application = self._create_v2i_application(application_sinks)
@@ -77,6 +76,10 @@ class Taxi(Node):
     def location(self) -> "Location":
         return self.mobility_model.location(self.env.now)
 
+    @location.setter
+    def location(self, value):
+        pass  # only for initialization, locations of this node is managed by the TaxiMobilityModel
+
     def _create_v2i_application(self, application_sinks: List[Node]) -> Application:
         application = Application()
         source_task = SourceTask(cu=0, bound_node=self)
