@@ -7,78 +7,83 @@ from leaf.infrastructure import Node, Link
 from leaf.power import PowerAware, PowerMeasurement
 
 
-class Task(PowerAware, ABC):
-    def __init__(self, mips: float):
-        """Task within an :class:`Application`, i.e. a node in the application DAG.
+class Task(PowerAware):
+    def __init__(self, cu: float):
+        """Task that can be placed on a :class:`Node`.
+
+        Tasks _can_ be connected via :class:`Link`s to build an :class:`Application`.
 
         Args:
-            mips: Million instructions per second required to execute the task.
+            cu: Amount of compute units (CU) required to execute the task. CUs a imaginary unit for computational
+                power to express differences between hardware platforms.
+
+            Million instructions per second required to execute the task.
         """
         self.id: Optional[int] = None
-        self.mips = mips
+        self.cu = cu
         self.node: Optional[Node] = None
 
     def __repr__(self):
-        return f"{self.__class__.__name__}(id={self.id}, mips={self.mips})"
+        return f"{self.__class__.__name__}(id={self.id}, cu={self.cu})"
 
     def allocate(self, node: Node):
         """Place the task on a certain node and allocate resources."""
         if self.node is not None:
             raise ValueError(f"Cannot place {self} on {node}: It was already placed on {self.node}.")
         self.node = node
-        node.add_task(self)
+        self.node._add_task(self)
 
     def deallocate(self):
         """Detache the task from the node it is currently placed on and deallocate resources."""
         if self.node is None:
             raise ValueError(f"{self} is not placed on any node.")
-        self.node.remove_task(self)
+        self.node._remove_task(self)
         self.node = None
 
     def measure_power(self) -> PowerMeasurement:
         try:
-            return self.node.measure_power().multiply(self.mips / self.node.used_mips)
+            return self.node.measure_power().multiply(self.cu / self.node.used_cu)
         except ZeroDivisionError:
             return PowerMeasurement(0, 0)
 
 
 class SourceTask(Task):
-    def __init__(self, mips: float = 0, bound_node: Node = None):
+    def __init__(self, cu: float = 0, bound_node: Node = None):
         """Source task of an application that is bound to a certain node, e.g. a sensor generating data.
 
         Source tasks never have incoming and always have outgoing data flows.
 
         Args:
-            mips: Million instructions per second required to execute the task.
+            cu: Million instructions per second required to execute the task.
             bound_node: The node which the task is bound to. Cannot be None.
         """
-        super().__init__(mips)
+        super().__init__(cu)
         if bound_node is None:
             raise ValueError("bound_node for SourceTask cannot be None")
         self.bound_node = bound_node
 
 
 class ProcessingTask(Task):
-    def __init__(self, mips: float = 0):
+    def __init__(self, cu: float = 0):
         """Processing task of an application that can be freely placed on the infrastructure.
 
         Processing tasks always have incoming and outgoing data flows.
 
         Args:
-            mips: Million instructions per second required to execute the task.
+            cu: Million instructions per second required to execute the task.
         """
-        super().__init__(mips)
+        super().__init__(cu)
 
 
 class SinkTask(Task):
-    def __init__(self, mips: float = 0, bound_node: Node = None):
+    def __init__(self, cu: float = 0, bound_node: Node = None):
         """Sink task of an application that is bound to a certain node, e.g. a cloud server for storage.
 
         Args:
-            mips: Million instructions per second required to execute the task.
+            cu: Million instructions per second required to execute the task.
             bound_node: The node which the task is bound to. Cannot be None.
         """
-        super().__init__(mips)
+        super().__init__(cu)
         if bound_node is None:
             raise ValueError("bound_node for SourceTask cannot be None")
         self.bound_node = bound_node
@@ -103,17 +108,19 @@ class DataFlow(PowerAware):
             raise ValueError(f"Cannot place {self} on {links}: It was already placed on path {self.links}.")
         self.links = links
         for link in self.links:
-            link.add_data_flow(self)
+            link._add_data_flow(self)
 
     def deallocate(self):
         """Remove the data flow from the infrastructure and deallocate bandwidth."""
         if self.links is None:
             raise ValueError(f"{self} is not placed on any link.")
         for link in self.links:
-            link.remove_data_flow(self)
+            link._remove_data_flow(self)
         self.links = None
 
     def measure_power(self) -> PowerMeasurement:
+        if self.links is None:
+            raise RuntimeError("Cannot measure power: DataFlow was not placed on any links.")
         return PowerMeasurement.sum(link.measure_power().multiply(self.bit_rate / link.used_bandwidth)
                                     for link in self.links)
 
