@@ -4,12 +4,14 @@ from typing import List, Optional, Type, TypeVar, Iterator, Union, Tuple
 import networkx as nx
 
 from leaf.power import PowerAware, PowerMeasurement
+from leaf.mobility import Location
 
 
 class Node(PowerAware):
     def __init__(self, name: str,
                  cu: Optional[float] = None,
-                 power_model: Optional["PowerModelNode"] = None):
+                 power_model: Optional["PowerModelNode"] = None,
+                 location: Optional[Location] = None):
         """A compute node in the infrastructure graph.
 
         This can represent any kind of node, e.g.
@@ -23,6 +25,7 @@ class Node(PowerAware):
             cu: Maximum processing power the node provides in "compute units", a imaginary unit for computational power
                 to express differences between hardware platforms. If None, the node has unlimited processing power.
             power_model: Power model which determines the power usage of the node.
+            location: The (x,y) coordinates of the node
         """
         self.name = name
         if cu is None:
@@ -39,6 +42,8 @@ class Node(PowerAware):
             self.power_model = power_model
             self.power_model.set_parent(self)
 
+        self.location = location
+
     def __repr__(self):
         cu_repr = self.cu if self.cu is not None else "∞"
         return f"{self.__class__.__name__}('{self.name}', cu={self.used_cu}/{cu_repr})"
@@ -70,7 +75,7 @@ class Node(PowerAware):
     def measure_power(self) -> PowerMeasurement:
         try:
             return self.power_model.measure()
-        except TypeError:
+        except AttributeError:
             raise RuntimeError(f"{self} has no power model.")
 
     def _reserve_cu(self, cu: float):
@@ -136,7 +141,7 @@ class Link(PowerAware):
     def measure_power(self) -> PowerMeasurement:
         try:
             return self.power_model.measure()
-        except TypeError:
+        except AttributeError:
             raise RuntimeError(f"{self} has no power model.")
 
     def _reserve_bandwidth(self, bandwidth):
